/*
 * Isprogrammer
 * This Arduino program is designed as USB interface between a PC-based ISp Programmer and atmel processor 89s51,89s52 and 89S8253.
 * The main program on PC sends serial commands to arduino, who send it as SPI to the Atmel processor
 * 10-10-2011  Karsten Albrektsen kaal@privat.dk
 *
 */

          
int rstPin = 8;
int sckPin = 9;
int misoPin = 10;
int mosiPin = 11;
unsigned long progPos =0;


boolean rst = false;
boolean sck = false;
boolean mosi = false;

unsigned long PROGRAM_ENABLE =       0xAC53FF69L;
unsigned long WRITE_PROGRAM =        0x40000000L;
unsigned long READ_PROGRAM =         0x20000000L;


unsigned long const ChipType_1_Read_Signature1_Bytes = 0x28000000L;
unsigned long const ChipType_1_Read_Signature2_Bytes = 0x28010000L;
unsigned long const ChipType_1_Read_Signature3_Bytes = 0x28020000L;

unsigned long const ChipType_2_Read_Signature1_Bytes = 0x28003000L;
unsigned long const ChipType_2_Read_Signature2_Bytes = 0x28003100L;
unsigned long const ChipType_2_Read_Signature3_Bytes = 0x28003200L;
unsigned long CHIP_ERASE =           0xAC80FFFFL;




char receivedChars[20];
int receivedCharsIndex =0;
boolean done = false;
int clockType = 0;


unsigned long data =0;
unsigned long adresse=0;



void setup()                    
{
  pinMode(rstPin, OUTPUT); 
  pinMode(sckPin, OUTPUT); 
  pinMode(mosiPin, OUTPUT); 
  pinMode(misoPin, INPUT);     

  digitalWrite(rstPin, LOW);
  digitalWrite(sckPin, HIGH);
  digitalWrite(mosiPin, HIGH);    
  digitalWrite(misoPin, HIGH);    

  Serial.begin(9600); 
}



int detectClocktype(){
  unsigned long result =0;
  int lastbyte  =0;


 progEnable(1);
 result=  clockCommand1(ChipType_1_Read_Signature1_Bytes);
 lastbyte = result % 256;
 if(lastbyte == 0x1e){
    clockType = 1;
    return lastbyte;
 }

  

 progEnable(2);
 result =  clockCommand2(ChipType_2_Read_Signature1_Bytes);
 lastbyte = result % 256;
 if(lastbyte == 0x1e){
    clockType = 2;
    return lastbyte;
 }


  return lastbyte;
  
}


unsigned long clockCommand(unsigned long command){

  if(clockType == 1){
   return clockCommand1(command);
}
  
if(clockType == 2){
   return clockCommand2(command);
}
return -1;

}




// 31 binalry ciffer is 10 ciffer i tital
/*
    89s52
*/


unsigned long clockCommand1(unsigned long command)
{


  unsigned long inBuf;

  unsigned long readBack = 0x0L;
  unsigned char inbyte;
  int mostSigBit;
  int i;


  mostSigBit = 31;
  for (i = mostSigBit; i >= 0; i--)  {

    if ((command >> i) & 0x01L)
      digitalWrite(mosiPin, HIGH);
    else
      digitalWrite(mosiPin, LOW);

    digitalWrite(sckPin, LOW); 

//    delay(1)  ;
    digitalWrite(sckPin, HIGH); 
//    delay(1)  ;

    inBuf =   digitalRead(misoPin);
    inBuf = (inBuf & 0x00000001L) << i;
    readBack = readBack | inBuf;
  }

  digitalWrite(sckPin, HIGH); 
  return(readBack);

}

// 31 binalry ciffer is 10 ciffer i tital
/*
    89s8253
*/


unsigned long clockCommand2(unsigned long command)
{


  unsigned long inBuf;

  unsigned long readBack = 0x0L;
  unsigned char inbyte;
  int mostSigBit;
  int i;


  mostSigBit = 31;
  for (i = mostSigBit; i >= 0; i--)  {
    

    digitalWrite(sckPin, HIGH); 
   // delay(1);
    

    if ((command >> i) & 0x01L)
      digitalWrite(mosiPin, HIGH);
    else
      digitalWrite(mosiPin, LOW);


    inBuf =   digitalRead(misoPin);
    inBuf = (inBuf & 0x00000001L) << i;
    readBack = readBack | inBuf;

    digitalWrite(sckPin, LOW); 
    digitalWrite(sckPin, HIGH); 


  }

  return(readBack);

}



void chipErase(){
    clockCommand(CHIP_ERASE);
}




int getChipSignatur(){


    unsigned long readBack = 0;
    unsigned char manufactureCode;
    unsigned char chipNameCode;
    unsigned char nameCode;
    
    digitalWrite(rstPin, HIGH);
    delay(20) ;
    readBack = clockCommand(ChipType_1_Read_Signature2_Bytes);
    int lastbyte =readBack % 256;
    return lastbyte;

}




int  readFlashCount()
{

    int chars =0;
    int n =0;
    int FFCount =0;

    unsigned long readBack = 0;
    while(FFCount < 6)
    {
        readBack = clockCommand(READ_PROGRAM + n*0x100);
        int lastbyte =readBack % 256;
        if(lastbyte == 0xFF)
        {
            FFCount++;
        }
        else
        {
            FFCount =0;
        }
        chars++;
        n++;
    }


    return n;

}




int readflash(unsigned long address){
     unsigned long readBack = 0;
     readBack = clockCommand(READ_PROGRAM + address*0x100);
     int lastbyte =readBack % 256;
     return lastbyte;
}


void writeflash(unsigned long data,unsigned long address)
{


  address=(0x00000000 |address)<<8;
  WRITE_PROGRAM= (address | WRITE_PROGRAM);

  WRITE_PROGRAM=(WRITE_PROGRAM | data);

  clockCommand(WRITE_PROGRAM);

 // delay(2);
  WRITE_PROGRAM=WRITE_PROGRAM & 0xFF000000;//to avoid adding up of previous address
}



int progEnable(int ct){


  unsigned long readBack = 0;
  int tries = 0;

  digitalWrite(mosiPin, LOW);
  digitalWrite(sckPin, LOW);
  digitalWrite(rstPin, LOW);

  delay(30);

  digitalWrite(rstPin, HIGH);

  delay(20);

  if(ct == 0)
     readBack = clockCommand(PROGRAM_ENABLE);
  if(ct == 1)
    readBack = clockCommand1(PROGRAM_ENABLE);
  if(ct == 2)
    readBack = clockCommand2(PROGRAM_ENABLE);

  readBack = (readBack & 0x000000FFL);//looking for return byte as of AVR

  return readBack;


}




boolean isNumber(char ch){

  return (ch >='0'  && ch <= '9');

}




unsigned long receiveLong(){
  done = false;
  receivedCharsIndex=0;
  while(! done){
    while (Serial.available() ==0);
    char incomingByte = Serial.read();
    if( ! isNumber(incomingByte) ){
      done = true;
    }
    else{
      receivedChars[receivedCharsIndex] = incomingByte;
      receivedCharsIndex++;
    }
  }
  receivedChars[receivedCharsIndex] = 0;

  return atol(receivedChars); 

}




void getSerialCommand(){

  if (Serial.available() > 0) {
    char incomingByte = Serial.read();

    if(incomingByte == 'a'){
      adresse= receiveLong();
      writeflash( data, adresse);
     // Serial.print(adresse);
      Serial.print("0a");
      return; 

    }
    
    
    
   


    if(incomingByte == 'e'){
      chipErase();
      Serial.print("0d");
      return; 
    }


    if(incomingByte == 'd'){
      data= receiveLong();
      writeflash( data, progPos);
      //Serial.print(data);
      progPos++;
      Serial.print("0d");
      return; 
    }



    if(incomingByte == 'g'){
        int b = readflash(progPos);
        progPos++;
        Serial.print(b);
        Serial.print("g");
        return;
    }



    if(incomingByte == 'p'){
      int detectedType = detectClocktype();
      //int responce = progEnable();      
      Serial.print(clockType);
      Serial.print('r');
      return; 
    }

    if(incomingByte == 's'){
      int responce = getChipSignatur();
      Serial.print(responce);
      Serial.print('r');
      return; 
    }


    if(incomingByte == 'b'){
      int responce = readFlashCount();
      progPos =0;
      Serial.print(responce);
      Serial.print('r');
      return; 
    }






    if(incomingByte == 'w'){
      progPos =0;
//      writeflash( data, adresse);
      //Serial.print(data);
      Serial.print("0w");
      return; 
    }
    

    if(incomingByte == 'c'){
      unsigned long command= receiveLong();
      int result = clockCommand(command);
      Serial.print( result);
      Serial.print('r');
      return; 
    }
    

    if(incomingByte == 'r'){
        digitalWrite(mosiPin, LOW);
        digitalWrite(sckPin, LOW);
        digitalWrite(rstPin, LOW);
        Serial.print("0r");
      return; 
    }
    
    
    if(incomingByte == 't'){
      Serial.print(clockType);         
      Serial.print("t");
     return; 
    }    
    


    if(incomingByte == 1){
      Serial.print("1z");
     return; 
    }    
    if(incomingByte == 0){
      Serial.print("0z");
     return; 
    }    



  }


}


void loop()                     
{
  getSerialCommand();
}


